import { Body, Controller, Get, Inject, Param, Query } from '@midwayjs/decorator';
import { BaseController } from 'free-midway-component';
import { RoomService } from '../service/room.service';
import { CreateDTO_room, CreateVO_room, FetchVO_room, QueryDTO_room, QueryVO_room, UpdateDTO_room, UpdateVO_room } from '../interface/room.interface';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
@ApiTags(['房间表'])
@Controller('/room')
export class RoomController extends BaseController {
  @Inject()
  service: RoomService;

  // @Post('/')
  @Validate()
  @ApiResponse({ type: CreateVO_room })
  async create(@Body() data: CreateDTO_room) {
    const result = await this.service.create(data);
    return result;
  }
  @Get('/')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QueryVO_room })
  async query(@Query() filter: QueryDTO_room, @Query('skip') skip: number, @Query('limit') limit: number) {
    const { data, total } = await this.service.newQuery(filter, { skip, limit, sort: { 'meta.createdAt': -1 } });
    const list = [];
    for (const i of data) {
      let nd = await this.service.fillId(i);
      nd = await this.service.countNotRead(filter, nd);
      list.push(nd);
    }
    return { data: list, total };
  }

  @Get('/:id')
  @ApiResponse({ type: FetchVO_room })
  async fetch(@Param('id') id: string) {
    const data = await this.service.fetch(id);
    const result = new FetchVO_room(data);
    return result;
  }

  // @Post('/:id')
  @Validate()
  @ApiResponse({ type: UpdateVO_room })
  async update(@Param('id') id: string, @Body() body: UpdateDTO_room) {
    const result = await this.service.updateOne(id, body);
    return result;
  }

  // @Del('/:id')
  @Validate()
  async delete(@Param('id') id: string) {
    await this.service.delete(id);
    return 'ok';
  }
  async createMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async updateMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async deleteMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }
}
