import { App, Provide, Config } from '@midwayjs/decorator';
import * as amqp from 'amqp-connection-manager';
import { Application } from '@midwayjs/koa';
import _ = require('lodash');
import { ChannelWrapper } from 'amqp-connection-manager';

@Provide()
export class MqSender {
  @App()
  app: Application;

  private connection: amqp.AmqpConnectionManager;

  private channelWrapper: ChannelWrapper;

  @Config('rabbitmq.url')
  mqUrl: object;

  @Config('mqConfig.normal')
  mqConfig: object;

  @Config('mqConfig.timeout')
  timeout: number;
  /**
   * 发送实时提示
   * 接收人需要有该提示
   * App的情况,应该是一个手机只能存在一个webstock的连接,所以需要区分属于哪部分,应该如何处理
   * @param receiver 接收人
   * @param data 数据
   */
  async toSendMsg(receiver, data) {
    await this.connect(receiver, data);
    // const res = await this.sendMsg(receiver, data);
    // console.log(res);
    await this.close();
  }

  async connect(queue, data) {
    // 创建连接，你可以把配置放在 Config 中，然后注入进来
    this.connection = await amqp.connect(this.mqUrl);
    // 创建 channel
    this.channelWrapper = this.connection.createChannel({
      json: true,
      setup: channel => {
        return Promise.all([
          // 绑定队列
          channel.assertExchange(_.get(this.mqConfig, 'ex'), 'direct', { durable: true }),
          channel.assertQueue(queue, { durable: false, exclusive: false }),
          channel.bindQueue(queue, _.get(this.mqConfig, 'ex')),
          channel.publish(_.get(this.mqConfig, 'ex'), queue, { ...data, type: 'chat' })
        ]);
      },
    });
  }

  async sendMsg(q: string, data: object) {
    return this.channelWrapper.publish(_.get(this.mqConfig, 'ex'), q, { ...data, type: 'chat' });
  }

  async close() {
    await this.channelWrapper.close();
    await this.connection.close();
  }
}
